@@ -1,5 +1,5 @@
 /* gsasl.h --- Header file for GNU SASL Library.
- * Copyright (C) 2002-2019 Simon Josefsson
+ * Copyright (C) 2002-2021 Simon Josefsson
  *
  * This file is part of GNU SASL Library.
  *
@@ -51,7 +51,7 @@ extern "C"
    * version number.  Used together with gsasl_check_version() to
    * verify header file and run-time library consistency.
    */
-#define GSASL_VERSION "1.8.1"
+#define GSASL_VERSION "1.10.0"
 
   /**
    * GSASL_VERSION_MAJOR
@@ -73,7 +73,7 @@ extern "C"
    *
    * Since: 1.1
    */
-#define GSASL_VERSION_MINOR 8
+#define GSASL_VERSION_MINOR 10
 
   /**
    * GSASL_VERSION_PATCH
@@ -84,7 +84,7 @@ extern "C"
    *
    * Since: 1.1
    */
-#define GSASL_VERSION_PATCH 1
+#define GSASL_VERSION_PATCH 0
 
   /**
    * GSASL_VERSION_NUMBER
@@ -95,7 +95,7 @@ extern "C"
    *
    * Since: 1.1
    */
-#define GSASL_VERSION_NUMBER 0x010801
+#define GSASL_VERSION_NUMBER 0x010a00
 
   /* RFC 2222: SASL mechanisms are named by strings, from 1 to 20
    * characters in length, consisting of upper-case letters, digits,
@@ -305,8 +305,11 @@ extern "C"
    * @GSASL_QOP: Quality-of-protection value.
    * @GSASL_SCRAM_ITER: Number of iterations in password-to-key hashing.
    * @GSASL_SCRAM_SALT: Salt for password-to-key hashing.
-   * @GSASL_SCRAM_SALTED_PASSWORD: Pre-computed salted SCRAM key,
-   *   to avoid re-computation and storing passwords in the clear.
+   * @GSASL_SCRAM_SALTED_PASSWORD: Hex-encoded hashed/salted password.
+   * @GSASL_SCRAM_SERVERKEY: Hex-encoded SCRAM ServerKey derived
+   *   from users' passowrd.
+   * @GSASL_SCRAM_STOREDKEY: Hex-encoded SCRAM StoredKey derived
+   *   from users' passowrd.
    * @GSASL_CB_TLS_UNIQUE: Base64 encoded tls-unique channel binding.
    * @GSASL_SAML20_IDP_IDENTIFIER: SAML20 user IdP URL.
    * @GSASL_SAML20_REDIRECT_URL: SAML 2.0 URL to access in browser.
@@ -346,6 +349,8 @@ extern "C"
     GSASL_SCRAM_ITER = 15,
     GSASL_SCRAM_SALT = 16,
     GSASL_SCRAM_SALTED_PASSWORD = 17,
+    GSASL_SCRAM_SERVERKEY = 23,
+    GSASL_SCRAM_STOREDKEY = 24,
     GSASL_CB_TLS_UNIQUE = 18,
     GSASL_SAML20_IDP_IDENTIFIER = 19,
     GSASL_SAML20_REDIRECT_URL = 20,
@@ -438,7 +443,7 @@ extern "C"
 					   Gsasl_session ** sctx);
   extern GSASL_API int gsasl_step (Gsasl_session * sctx,
 				   const char *input, size_t input_len,
-				   char **output, size_t * output_len);
+				   char **output, size_t *output_len);
   extern GSASL_API int gsasl_step64 (Gsasl_session * sctx,
 				     const char *b64input, char **b64output);
   extern GSASL_API void gsasl_finish (Gsasl_session * sctx);
@@ -446,10 +451,10 @@ extern "C"
   /* Session functions: xcode.c, mechname.c */
   extern GSASL_API int gsasl_encode (Gsasl_session * sctx,
 				     const char *input, size_t input_len,
-				     char **output, size_t * output_len);
+				     char **output, size_t *output_len);
   extern GSASL_API int gsasl_decode (Gsasl_session * sctx,
 				     const char *input, size_t input_len,
-				     char **output, size_t * output_len);
+				     char **output, size_t *output_len);
   extern GSASL_API const char *gsasl_mechanism_name (Gsasl_session * sctx);
 
   /* Error handling: error.c */
@@ -461,26 +466,55 @@ extern "C"
 				       Gsasl_saslprep_flags flags, char **out,
 				       int *stringpreprc);
 
-  /* Utilities: base64.c, md5pwd.c, crypto.c */
+  /* Crypto functions: crypto.c */
+  typedef enum
+  {
+    /* Hash algorithm identifiers. */
+    GSASL_HASH_SHA1 = 2,
+    GSASL_HASH_SHA256 = 3,
+  } Gsasl_hash;
+
+  typedef enum
+  {
+    /* Output sizes of hashes. */
+    GSASL_HASH_SHA1_SIZE = 20,
+    GSASL_HASH_SHA256_SIZE = 32,
+    GSASL_HASH_MAX_SIZE = GSASL_HASH_SHA256_SIZE
+  } Gsasl_hash_length;
+
+  extern GSASL_API int gsasl_nonce (char *data, size_t datalen);
+  extern GSASL_API int gsasl_random (char *data, size_t datalen);
+
+  extern GSASL_API size_t gsasl_hash_length (Gsasl_hash hash);
+
+  extern GSASL_API int
+    gsasl_scram_secrets_from_salted_password (Gsasl_hash hash,
+					      const char *salted_password,
+					      char *client_key,
+					      char *server_key,
+					      char *stored_key);
+  extern GSASL_API int
+    gsasl_scram_secrets_from_password (Gsasl_hash hash,
+				       const char *password,
+				       unsigned int iteration_count,
+				       const char *salt,
+				       size_t saltlen,
+				       char *salted_password,
+				       char *client_key,
+				       char *server_key, char *stored_key);
+
+  /* Utilities: md5pwd.c, base64.c, free.c */
   extern GSASL_API int gsasl_simple_getpass (const char *filename,
 					     const char *username,
 					     char **key);
   extern GSASL_API int gsasl_base64_to (const char *in, size_t inlen,
-					char **out, size_t * outlen);
+					char **out, size_t *outlen);
   extern GSASL_API int gsasl_base64_from (const char *in, size_t inlen,
-					  char **out, size_t * outlen);
-  extern GSASL_API int gsasl_nonce (char *data, size_t datalen);
-  extern GSASL_API int gsasl_random (char *data, size_t datalen);
-  extern GSASL_API int gsasl_md5 (const char *in, size_t inlen,
-				  char *out[]);
-  extern GSASL_API int gsasl_hmac_md5 (const char *key, size_t keylen,
-				       const char *in, size_t inlen,
-				       char *outhash[]);
-  extern GSASL_API int gsasl_sha1 (const char *in, size_t inlen,
-				   char *out[]);
-  extern GSASL_API int gsasl_hmac_sha1 (const char *key, size_t keylen,
-					const char *in, size_t inlen,
-					char *outhash[]);
+					  char **out, size_t *outlen);
+  extern GSASL_API int gsasl_hex_to (const char *in, size_t inlen,
+				     char **out, size_t *outlen);
+  extern GSASL_API int gsasl_hex_from (const char *in, char **out,
+				       size_t *outlen);
   extern GSASL_API void gsasl_free (void *ptr);
 
   /* Get the mechanism API. */
